import bpy, traceback
from bpy.app.handlers import persistent
from .camera import update_settings, update_world
from .properties.post_effects import update_post_effects

LAST_CAMERA = None
LAST_SENSOR_WIDTH = 1
LAST_SENSOR_HEIGHT = 1
LAST_WORLD = None

@persistent
def update_scene_camera(scene):
    global LAST_CAMERA
    global LAST_SENSOR_WIDTH
    global LAST_SENSOR_HEIGHT
    global LAST_WORLD
    if scene.camera:
        if scene.camera != LAST_CAMERA or scene.camera.data.sensor_width != LAST_SENSOR_WIDTH or scene.camera.data.sensor_height != LAST_SENSOR_HEIGHT:
            LAST_CAMERA = scene.camera
            LAST_SENSOR_WIDTH = scene.camera.data.sensor_width
            LAST_SENSOR_HEIGHT = scene.camera.data.sensor_height
            update_settings(scene.camera.data.photographer,bpy.context)
            update_post_effects(scene.camera.data.post_effects,bpy.context)

        try:
            if scene.world != LAST_WORLD:
                LAST_WORLD = scene.world
                update_world(scene.camera.data.photographer,bpy.context)
                # raise Exception('Photographer Exception')
        except Exception as e:
            print(f'Photographer Handler {type(e)}: {e} {traceback.format_exc()}')

def register():
    bpy.app.handlers.depsgraph_update_post.append(update_scene_camera)

def unregister():
    bpy.app.handlers.depsgraph_update_post.remove(update_scene_camera)
